@@ -2,6 +2,7 @@
 #include <assert.h>
 #include <string.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include "tool.h"
 #include "link.h"
 #include "kernelsyms.h"
@@ -21,7 +22,7 @@ static int depmod_addksyms (
 	int nbsym=1;
 	// Fake the _mod_use_count_ symbol which is provided by insmod
 	int requis;		// Dummy
-	tbsym[0] = syms.add ("_mod_use_count_",mod,SYM_DEFINI,requis,0);
+	tbsym[0] = syms.add ("mod_use_count_",mod,SYM_DEFINI,requis,0);
 #if 0
 		char *cmd_strings = "strings /usr/src/linux/kernel/ksyms.o";
 		FILE *fin = popen_err (cmd_strings,"r",0);
@@ -44,7 +45,7 @@ static int depmod_addksyms (
 		int kernel_syms = 0;
 		/* #Specification: depmod / kernel syms only
 			When initialising its symbol table from the kernel
-			depmod silently discad all symbol from loaded modules.
+			depmod silently discards all symbol from loaded modules.
 
 			This means that depmod may be used at any time to compute
 			the dependancy table, even if there are modules already
@@ -55,18 +56,25 @@ static int depmod_addksyms (
 			kernel symbols are at the end of the list, just
 			after a pseudo symbol with a one character name: #
 		*/
+		int a_out_kernel = 0;
 		for (ksym = ksymtab; so_far < nksyms ; ++so_far, ksym++) {
 			if (kernel_syms){
+				if (ksym->name[0] == '#')
+					continue;
 				assert (nbsym < 2000);
-				tbsym[nbsym++] = syms.add (ksym->name,mod,SYM_DEFINI,requis,0);
+				// If a_out kernel, skip leading '_'
+				tbsym[nbsym++] = syms.add (ksym->name + a_out_kernel,mod,SYM_DEFINI,requis,0);
 			}else if (strcmp(ksym->name,"#") == 0){
 				kernel_syms = 1;
+				// This works... I promise!
+				if ((ksym + 1)->name[0] == '_')
+					a_out_kernel = 1;
 			}
 		}
 #else
 		FILE *fin = fopen ("/proc/ksyms","r");
 		if (fin == NULL){
-			fprintf (stderr,"Can't open /proc/ksyms\n");
+			depmod_error ("Can't open /proc/ksyms");
 		}else{
 			char buf[300];
 			char lin[300];
@@ -83,7 +91,7 @@ static int depmod_addksyms (
 	int size = nbsym*sizeof(SYMBOL*);
 	mod->pub.tb = (SYMBOL**)malloc(size);
 	if (mod->pub.tb == NULL){
-		fprintf (stderr,"Can't allocate memory for kernel symbols\n");
+		depmod_error ("Can't allocate memory for kernel symbols");
 	}else{
 		memcpy (mod->pub.tb,tbsym,size);
 		ret = 0;
@@ -103,17 +111,20 @@ static int depmod_load (MODULES &mods, SYMBOLS &syms, const char *path)
 */
 static void depmod_prtdepend(
 	MODULES &mods,
-	const char *depfile)
+	const char *depfile,
+	int verbose,
+	int showerror)
 {
 	FILE *out = stdout;
 	if (depfile != NULL){
 		out = fopen (depfile,"w");
 		if (out == NULL){
-			fprintf (stderr,"Can't open %s\n",depfile);
+			depmod_error ("Can't open %s",depfile);
 			exit (-1);
 		}
 	}								
-	mods.prtdepend(out,"-");
+	mods.prtdepend(out,"-",verbose,showerror);
+
 }
 
 int depmod_main (int argc, char *argv[])
@@ -122,46 +133,83 @@ int depmod_main (int argc, char *argv[])
 	if (argc == 1){
 		fprintf (stderr,
 			"depmod " DEPMOD_RELEASE "\n"
-			"depmod -a\n"
-			"depmod module1.o module2.o ...\n"
+			"depmod [-d -e -v ] -a\n"
+			"depmod [-d -e -v ] module1.o module2.o ...\n"
 			"\n"
 			"depmod will output a dependancy list suitable for the\n"
 			"modprobe utility. This dependancy file look like\n"
 			"a Makefile\n"
 			"\n"
-			// "It reads the file /proc/ksyms\n"
-			"It reads the kernel symbol table\n"
-			"to find out which symbols are already available\n"
-			"in the running kernel\n"
-			"\n"
-			"It will output on stdout\n"
+			"It reads the kernel symbol table to find out which\n"
+			"symbols are already available in the running kernel.\n"
 			"\n"
 			"depmod -a will find the list of module to probe from\n"
-			"the file " ETC_CONF_MODULES "\n"
-			"It will output the result into the depfile specified\n"
-			"in this configuration file\n"
+			"the file " ETC_CONF_MODULES ". It will output the result\n"
+			"into the depfile specified in this configuration file\n"
 			"(depfile=...)\n"
+			"\n"
+			"Normally depmod operate silently, reporting only the list\n"
+			"of module that won't load properly (missing symbols).\n"
+			"Option -e output all the unresolved symbol for a given module\n"
+			"Option -s output all error message to the syslog daemon\n"
+			"Option -v force a printout of all visited modules.\n"
 			);
 	}else{
 		MODULES mods;
 		SYMBOLS syms;
 		ret = depmod_addksyms(mods,syms);
 		if (ret != -1){
-			if (argc == 2 && strcmp(argv[1],"-a")==0){
-				if (config_read() == -1){
+			int opt;
+			int showerror = 0;
+			int verbose = 0;
+			int stdmode = 0;
+			int err = 0;
+			extern int optind;
+			while ((opt=getopt(argc,argv,"adesv"))!=-1){
+				switch(opt){
+				case 'a':
+					stdmode = 1;	// Probe standard directory
+									// using the config file
+					break;
+				case 'd':
+					debugmode = 1;
+					break;
+				case 'e':
+					showerror = 1;
+					break;
+				case 's':
+					depmod_setsyslog ("modprobe");
+					break;
+				case 'v':
+					verbose = 1;
+					break;
+				case '?':
+					err = 1;
+					break;
+				}
+			}
+			if (err){
+				depmod_error ("Aborting");
+				ret = -1;
+			}else if (stdmode){
+				if (optind != argc){
+					depmod_error ("No argument with option -a. Aborting");
+					ret = -1;
+				}else if (config_read() == -1){
 					ret = -1;
 				}else{
 					char *lst[1000];
-					int nb = config_lstmod ("*.{mod,o}",NULL,lst);
+					int nb = config_lstmod ("*.{mod,o}",NULL,lst,1);
 					for (int i=0; i<nb && ret != -1; i++){
 						ret = depmod_load (mods,syms,lst[i]);
 					}
 					if (ret != -1){
-						depmod_prtdepend (mods,config_getdepfile());
+						depmod_prtdepend (mods,config_getdepfile()
+							,verbose,showerror);
 					}
 				}				
 			}else{
-				for (int i=1; i<argc && ret != -1; i++){
+				for (int i=optind; i<argc && ret != -1; i++){
 					char *arg = argv[i];
 					if (arg[0] == '-'){
 						// Option processing, one day ...
@@ -170,7 +218,8 @@ int depmod_main (int argc, char *argv[])
 					}
 				}
 				if (ret != -1){
-					depmod_prtdepend(mods,NULL);
+					depmod_prtdepend(mods,NULL,verbose,showerror);
+
 				}
 			}
 		}
